#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3			// Use modern global access method and strict wave access.
#pragma IgorVersion=9.0	// requires in-line multiple return syntax
#include "AstronomicalAlgorithms"
#define webApps 			// comment this to disable web applications which can be slow

// menu
Menu "Tools"
	"Astronomical data",/Q,getAstronomicalDataPrompt()
end

// some defaults for latitude and longitude 
Static Constant defaultlatVar =  -7.9286077  // Obsidian Hotel, Georgetown
Static Constant defaultlonVar =  -14.4106611 // Obsidian Hotel, Georgetown

//___________________________________________________________________________________________________	
// Call popup of time, position, timezone, then display output in notebook
Function getAstronomicalDataPrompt()
	
	String datStr=ParseDateStringNum(replacestring("-",secs2date(datetime,-2),"")) // current date in YYYYMMDD
	String timeStr=secs2Time(datetime,3) // current local time 
	Variable latVar=defaultlatVar
	Variable lonVar=defaultlonVar
	String TimeZoneStr=KeyByNumber((date2secs(-1,-1,-1)/3600),TimeZoneKeyPairs(),keySepStr=",") // current UTC offset
	Variable altVar

	#ifdef webApps
		// get lat lon and UTC offset from an internet service 	
		[latVar,lonVar,Variable offset] = getApproxLocationTimeZone() // approximate location from IP address
		TimeZoneStr=KeyByNumber(offset/3600,TimeZoneKeyPairs(),keySepStr=",") // current UTC offset
	#endif	
	
	Prompt datStr "Enter date"
	Prompt timeStr "Enter time"
	Prompt latVar "Enter decimal latitude"
	Prompt lonVar "Enter decimal longitude"
	Prompt TimeZoneStr "Choose a time zone", popup getKeysfromKeyPairs(TimeZoneKeyPairs(),",",";")
	Prompt altVar "Enter the observer altitude in meters"
	
	// prompt user for data input with defaults and current time/date already input
	DoPrompt/HELP="" "Astronomical data" datStr, timeStr,latVar,lonVar,TimeZoneStr,altVar
		if (V_flag!=0) // user cancelled
			return 0 	
		endif
	
	// get date format	
	if (GrepString(datStr,"^[0-2][0-9][0-9][0-9][0-1][0-9][0-3][0-9]$")) // date yyyymmdd
	// do nothing
	elseif (GrepString(datStr,"^[0-2][0-9][0-9][0-9]-[0-1][0-9]-[0-3][0-9]$")) // date yyyy-mm-dd
		datStr=datStr[0,3]+datStr[5,6]+datStr[8,9]
	elseif (GrepString(datStr,"^[0-3][0-9]-[0-1][0-9]-[0-2][0-9][0-9][0-9]$")) // date dd-mm-yyyy
		datStr=datStr[6,9]+datStr[3,4]+datStr[0,1]
	elseif (GrepString(datStr,"^[0-2][0-9][0-9][0-9]-[A-Za-z][A-Za-z][A-Za-z]-[0-3][0-9]$")) // date yyyy-mmm-dd
		datStr=ParseDateStringAlpha(datstr)
	elseif (GrepString(datStr,"^[0-3][0-9]-[A-Za-z][A-Za-z][A-Za-z]-[0-2][0-9][0-9][0-9]$")) // date dd-mmm-yyyy
		datStr=ParseDateStringAlpha(datStr[7,9]+"-"+datStr[6,6]+"-"+datStr[0,1])		
	else
		DoAlert/T="Astronomical data" 0, "The format of the date input was not recognised" 
		Return 0
	endif
	
	// check the time format
	if (!GrepString(timeStr,"^[0-2][0-9]:[0-5][0-9]:[0-5][0-9]$"))
		DoAlert/T="Astronomical data" 0, "The time format was not recognised" 
		Return 0
	endif 
	
	Variable hourVar,minuteVar,SecondVar
	sscanf timeStr, "%f:%f:%f", hourVar,minuteVar,SecondVar
	
	if ((hourVar>23)|(minuteVar>59)|(secondVar>59))
		DoAlert/T="Astronomical data" 0, "The time entered is not a valid time" 
		Return 0
	endif 
	
	// call the calculations and display the output in a notebook	
	AstronomicalDataPanel(getAstronomicalData(datStr,timeStr,latVar,lonVar,NumberByKey(TimeZoneStr,TimeZoneKeyPairs(),",",";"),altVar=altVar))

end
//___________________________________________________________________________________________________	
Function AstronomicalDataPanel(String dataStr)

	// Get screen size
	Variable v0,v1,v2,v3,v4
	sscanf stringbykey("SCREEN1",IgorInfo(0)),"DEPTH=%i,RECT=%i,%i,%i,%i", v0,v1,v2,v3,v4
	
	// Get a central position for panel
	Variable left,top,right,bottom,grout
	left = (v3/2)-(v3/4)
	top = 0//(v4/2)-(v4/3)
	right = (v3/2)+(v3/4)
	bottom = (v4/2)+(v4/4)
	grout = 0.01
	
	// Make the output strings
	String nb1Str="",nb2Str="", tmpStr
	
	// get timezone offset 
	Variable offVar = NumberByKey("TimeZone",dataStr)/24
	
	// solar data
	sprintf tmpStr "" ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Day number: %d (of %d)",NumberByKey("DayNumber",dataStr),NumberByKey("DaysInYear",dataStr) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Julian Day: %.6f",NumberByKey("JulianDay",dataStr) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Ephemeris Time: %s",Julian2Time(NumberByKey("JulianEphemerisDay",dataStr),T=3) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "True Solar Time: %s", RA2HMS(mod(NumberByKey("SolarTime",dataStr),2*pi)) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "" ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Greenwich Mean Sidereal Time: %s",DD2HMS(NumberByKey("GMST",dataStr)) ; nb2str+=tmpStr+"\r"
//	sprintf tmpStr "Apparent Greenwich Sidereal Time: %s",DD2HMS(NumberByKey("GST",dataStr)) ; nb2str+=tmpStr+"\r"
//	sprintf tmpStr "Local Mean Sidereal Time: %s",DD2HMS(NumberByKey("LMST",dataStr)) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Apparent Local Sidereal Time: %s",DD2HMS(NumberByKey("LST",dataStr)) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "________________________________________________________" ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "" ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Astronomical dawn: %s - %s",Julian2Time(NumberByKey("AstronomicalTwilightStart",dataStr)+offVar,T=3),Julian2Time(NumberByKey("NauticalTwilightStart",dataStr)+offVar,T=3) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Nautical dawn: %s - %s",Julian2Time(NumberByKey("NauticalTwilightStart",dataStr)+offVar,T=3),Julian2Time(NumberByKey("CivilTwilightStart",dataStr)+offVar,T=3) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Civil dawn: %s - %s",Julian2Time(NumberByKey("CivilTwilightStart",dataStr)+offVar,T=3),Julian2Time(NumberByKey("SolarRise",dataStr)+offVar,T=3) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Sunrise: %s (%.3f° azimuth)",Julian2Time(NumberByKey("SolarRise",dataStr)+offVar,T=3),deg(NumberByKey("SolarRiseAzimuth",dataStr)) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Solar Noon: %s (%.3f° elevation)",Julian2Time(NumberByKey("SolarTransit",dataStr)+offVar,T=3),deg(NumberByKey("SolarTransitAltitude",dataStr)) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Golden hour: %s - %s",Julian2Time(NumberByKey("GoldenHourStart",dataStr),T=3),Julian2Time(NumberByKey("SolarSet",dataStr)+offVar,T=3) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Sunset: %s (%.3f° azimuth)",Julian2Time(NumberByKey("SolarSet",dataStr)+offVar,T=3),deg(NumberByKey("SolarSetAzimuth",dataStr)) ; nb2str+=tmpStr+"\r" 
	sprintf tmpStr "Civil twilight: %s - %s",Julian2Time(NumberByKey("SolarSet",dataStr)+offVar,T=3),Julian2Time(NumberByKey("CivilTwilightFinish",dataStr)+offVar,T=3) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Nautical twilight: %s - %s",Julian2Time(NumberByKey("CivilTwilightFinish",dataStr)+offVar,T=3),Julian2Time(NumberByKey("NauticalTwilightFinish",dataStr)+offVar,T=3) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Astronomical twilight: %s - %s",Julian2Time(NumberByKey("NauticalTwilightFinish",dataStr)+offVar,T=3),Julian2Time(NumberByKey("AstronomicalTwilightFinish",dataStr)+offVar,T=3) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Nadir: %s",Julian2Time(NumberByKey("Nadir",dataStr)+offVar,T=3) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Sunlight duration: %s (%s darkness)",Secs2Time(NumberByKey("DaylightDuration",dataStr),3),Secs2Time(NumberByKey("NightDuration",dataStr),3) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "" ; nb2str+=tmpStr+"\r"
//	sprintf tmpStr "Geometric Mean Longitude of the Sun: %.3f°",NumberByKey("SunMeanLong",dataStr) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "True Longitude of the Sun: %.3f°",NumberByKey("SunTrueLong",dataStr) ; nb2str+=tmpStr+"\r"
//	sprintf tmpStr "Geometric Mean Anomally of the Sun: %.3f°",mod(NumberByKey("SunMeanAnom",dataStr),360) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "True Anomally of the Sun: %.3f°",mod(NumberByKey("SunTrueAnom",dataStr),360) ; nb2str+=tmpStr+"\r"	
	sprintf tmpStr "Eccentricity of Earth's Orbit: %.4f",NumberByKey("EccentEarthOrbit",dataStr) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Obliquity of the Ecliptic: %.3f°",NumberByKey("ObliquityOfTheEcliptic",dataStr) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Equation of the Centre: %.4f",NumberByKey("EqnOfTheCentre",dataStr) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Equation of Time: %s", RA2HMS(NumberByKey("SolarEquationOfTime",dataStr)) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "" ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Right ascension: %s",RA2HMS(NumberByKey("SolarRightAscension",dataStr)) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Solar declination: %.3f°",deg(NumberByKey("SolarDeclination",dataStr)) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Solar zenith: %.3f°",NumberByKey("SolarZenith",dataStr) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Solar azimuth: %.3f°",deg(NumberByKey("SolarAzimuth",dataStr)) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Solar elevation: %.3f°",deg(NumberByKey("SolarAltitude",dataStr)) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Shadow Length: %s",shaddowLengthStr(NumberByKey("SolarShadow",dataStr),"Sun") ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "" ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Earth is %.2f million km from Sol %s at %.2f km/h",NumberByKey("SolarDistance",dataStr)/1000000000,directionStr(NumberByKey("SolarRadialSpeed",dataStr)),abs(NumberByKey("SolarRadialSpeed",dataStr))*3.6 ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Light from Sol takes %s to reach Earth",secs2time((NumberByKey("SolarDistance",dataStr)/299792458),3,3) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "" ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Vernal Equinox: %s",Julian2DateTime(NumberByKey("SpringEquin",dataStr)+offVar,D=1) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Summer Solstice: %s",Julian2DateTime(NumberByKey("SummerSolst",dataStr)+offVar,D=1) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Autumnal Equinox: %s",Julian2DateTime(NumberByKey("AutumnEquin",dataStr)+offVar,D=1) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Winter Solstice: %s",Julian2DateTime(NumberByKey("WinterSolst",dataStr)+offVar,D=1) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "" ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Longitude of Perihelion: %.3f°",NumberByKey("LongOfPerihelion",dataStr) ; nb2str+=tmpStr+"\r"
	sprintf tmpStr "Distance at Perihelion: %.2f million km",NumberByKey("DistanceOfPerihelion",dataStr)/1000000000 ; nb2str+=tmpStr+"\r"

	// lunar data
	sprintf tmpStr "Inputs:" ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "\tDate: %s",ParseDateStringNum(StringByKey("datStr",dataStr)) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "\tTime: %s %s",StringByKey("timeStr",dataStr),keybyNumber(NumberByKey("TimeZone",dataStr),TimeZoneKeyPairs(),keySepStr=",") ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "\tLatitude: %.4f (%s)",NumberByKey("latVar",dataStr),DDlat2DMS(NumberByKey("latVar",dataStr),verb=0) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "\tLongitude: %.4f (%s)",NumberByKey("lonVar",dataStr),DDlon2DMS(NumberByKey("lonVar",dataStr),verb=0) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "\tAltitude: %.1f m (%.1f ft)",NumberByKey("altVar",dataStr),NumberByKey("altVar",dataStr)*3.281 ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "________________________________________________________" ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "" ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Moon Age: %.3f days",NumberByKey("LunarAge",dataStr) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Moon Phase: %s",StringByKey("LunarPhase",dataStr) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Moon Illuminated Fraction: %.2f %%",NumberByKey("LunarIlluminatedFraction",dataStr)*100 ; nb1str+=tmpStr+"\r"   
	sprintf tmpStr "" ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Moonrise: %s (%.3f° azimuth)",Julian2DateTime(NumberByKey("LunarRise",dataStr)+offVar,T=3,D=1),deg(NumberByKey("LunarRiseAzimuth",dataStr)) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Lunar Transit: %s (%.3f° elevation)",Julian2DateTime(NumberByKey("LunarTransit",dataStr)+offVar,T=3,D=1),deg(NumberByKey("LunarTransitAltitude",dataStr)) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Moonset: %s (%.3f° azimuth)",Julian2DateTime(NumberByKey("LunarSet",dataStr)+offVar,T=3,D=1),deg(NumberByKey("LunarSetAzimuth",dataStr)) ; nb1str+=tmpStr+"\r" 
	sprintf tmpStr "" ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Lunar Longitude: %.3f°",NumberByKey("LunarLongitude",dataStr) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Lunar Latitude: %.3f°",NumberByKey("LunarLatitude",dataStr) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Center of Moon's bright limb: %.3f° clockwise from Zenith",mod(360 - deg(NumberByKey("LunarLimb",dataStr)),360) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Lunar Phase Angle: %.3f°",deg(NumberByKey("LunarPhaseAngle",dataStr)) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Sun-Moon anglar Separation: %.3f°",deg(NumberByKey("LunarSolarAngularSeparation",dataStr)) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "" ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Right ascension: %s",RA2HMS(NumberByKey("LunarRightAscension",dataStr)) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Lunar declination: %.3f°",deg(NumberByKey("LunarDeclination",dataStr)) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Lunar zenith: %.3f°",NumberByKey("LunarZenith",dataStr) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Lunar azimuth: %.3f°",deg(NumberByKey("LunarAzimuth",dataStr)) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Lunar elevation: %.3f°",deg(NumberByKey("LunarAltitude",dataStr)) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Shadow Length: %s",shaddowLengthStr(NumberByKey("LunarShadow",dataStr),"Moon") ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "" ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Luna is %.2f thousand km from Earth %s at %.2f km/h",NumberByKey("LunarDistance",dataStr)/1000000,directionStr(NumberByKey("LunarRadialSpeed",dataStr)),abs(NumberByKey("LunarRadialSpeed",dataStr))*3.6 ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Light from Luna takes %.3f seconds to reach Earth",NumberByKey("LunarDistance",dataStr)/299792458 ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "" ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "New Moon; %s",Julian2DateTime(NumberByKey("NewLune",dataStr)+offVar,D=1) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "First Quarter; %s",Julian2DateTime(NumberByKey("FirstQuarterLune",dataStr)+offVar,D=1) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Full Moon; %s",Julian2DateTime(NumberByKey("FullLune",dataStr)+offVar,D=1) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Last Quarter; %s",Julian2DateTime(NumberByKey("LastQuarterLune",dataStr)+offVar,D=1) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "" ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Lunar Ascending Node: %.3f°",NumberByKey("LunarAscendingNode",dataStr) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Date of Perigee: %s",Julian2DateTime(NumberByKey("LunarPerigee",dataStr)+offVar,D=1) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Distance at Perigee: %.2f thousand km",NumberByKey("LunarPerigeeDistance",dataStr)/1000000 ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Date of Apogee: %s",Julian2DateTime(NumberByKey("LunarApogee",dataStr)+offVar,D=1) ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "Distance at Apogee: %.2f thousand km",NumberByKey("LunarApogeeDistance",dataStr)/1000000 ; nb1str+=tmpStr+"\r"
	sprintf tmpStr "________________________________________________________" ; nb1str+=tmpStr+"\r"
	#ifdef webApps
		sprintf tmpStr "" ; nb1str+=tmpStr+"\r"
		sprintf tmpStr "Magnetic North Latitude: %.4f (%s)",NumberByKey("NPlatitude",dataStr),DDlat2DMS(NumberByKey("NPlatitude",dataStr),verb=0) ; nb1str+=tmpStr+"\r"
		sprintf tmpStr "Magnetic North Longitude: %.4f (%s)",NumberByKey("NPlongitude",dataStr),DDlon2DMS(NumberByKey("NPlongitude",dataStr),verb=0) ; nb1str+=tmpStr+"\r"
		sprintf tmpStr "" ; nb1str+=tmpStr+"\r"
		sprintf tmpStr "Magnetic South Latitude: %.4f (%s)",NumberByKey("SPlatitude",dataStr),DDlat2DMS(NumberByKey("SPlatitude",dataStr),verb=0) ; nb1str+=tmpStr+"\r"
		sprintf tmpStr "Magnetic South Longitude: %.4f (%s)",NumberByKey("SPlongitude",dataStr),DDlon2DMS(NumberByKey("SPlongitude",dataStr),verb=0) ; nb1str+=tmpStr+"\r"
	#endif	
	
	// Make sure any existing panel is killed
	KillWindow/Z AstroPanel

	// Make panel with notebooks in it
	NewPanel/K=1/N=AstroPanel/NA=2/W=(left,top,right,bottom) as "Astronomical data"
	NewNotebook/HOST=AstroPanel/N=nb1/F=1/OPTS=3/W=(0+grout,0+grout,0.5-(grout/2),1-grout)
	NewNotebook/HOST=AstroPanel/N=nb2/F=1/OPTS=3/W=(0.5+(grout/2),0+grout,1-grout,1-grout)
	
	Notebook AstroPanel#nb1,frameStyle=0, text=nb1Str	// insert text into notebook
	Notebook AstroPanel#nb2,frameStyle=0, text=nb2Str	// insert text into notebook
	
	Notebook AstroPanel#nb1 selection={startOfFile,startOfFile}, findText={"",1}		// go to the top of the notebook
	Notebook AstroPanel#nb2 selection={startOfFile,startOfFile}, findText={"",1}		// go to the top of the notebook
	
	Notebook AstroPanel#nb1 writeProtect=1	// prevent modification
	Notebook AstroPanel#nb2 writeProtect=1	// prevent modification
	
	SetActiveSubwindow AstroPanel
	DoWindow/F AstroPanel	// bring panel to front
end
//___________________________________________________________________________________________________	
// call solar geometric calculations and output as key/value pairs
Function/S getAstronomicalData(datStr,timeStr,latVar,lonVar,TimeZone[,altVar])
	String datStr,timeStr
	Variable latVar,lonVar,TimeZone,altVar
	
	// T + P constants 
	Variable temp = 283
	Variable pressure = 1010
	
	// make sure observer height is positive
	if ( altVar < 0 )
		altVar = 0
		endif
	
	// form the output data string
	String rtnStr="",tmpStr
	
	// output the inputs
	sprintf tmpStr,"datStr:%s;", datStr ; rtnStr+=tmpStr
	sprintf tmpStr,"timeStr:%s;", timeStr ; rtnStr+=tmpStr
	sprintf tmpStr,"latVar:%f;", latVar ; rtnStr+=tmpStr
	sprintf tmpStr,"lonVar:%f;", lonVar ; rtnStr+=tmpStr
	sprintf tmpStr,"altVar:%f;", altVar ; rtnStr+=tmpStr
	sprintf tmpStr,"TimeZone:%f;", TimeZone ; rtnStr+=tmpStr
	
	// solar calculations
	Variable JD = JulianDay(datStr,timeStr,TimeZone) ; sprintf tmpStr,"JulianDay:%f;",JD ; rtnStr+=tmpStr	// days since January 1, 4713 BC
	Variable DayNumber =  Julian2DayNumber(JD) ; sprintf tmpStr,"DayNumber:%f;",DayNumber ; rtnStr+=tmpStr // days since new year
	Variable DaysInYear = Julian2Days(JD) ; sprintf tmpStr,"DaysInYear:%f;",DaysInYear ; rtnStr+=tmpStr // days in year
	Variable JulianEphemerisDay = JDE(JD) ; sprintf tmpStr,"JulianEphemerisDay:%f;",JulianEphemerisDay ; rtnStr+=tmpStr	// days since January 1, 4713 BC
	Variable SolarTrueLongitude =  sunTrueLongitude(JD) ; sprintf tmpStr,"SunTrueLong:%f;",SolarTrueLongitude ; rtnStr+=tmpStr // degrees
	Variable SolarTrueAnomaly = sunTrueAnomaly(JD) ; sprintf tmpStr,"SunTrueAnom:%.4f;",SolarTrueAnomaly ; rtnStr+=tmpStr // degrees
	Variable SolarMeanLongitude =  sunGeometricMeanLongitude(JD) ; sprintf tmpStr,"SunMeanLong:%f;",SolarMeanLongitude ; rtnStr+=tmpStr // degrees
	Variable SolarMeanAnomaly = sunMeanAnomaly(JD) ; sprintf tmpStr,"SunMeanAnom:%f;",SolarMeanAnomaly ; rtnStr+=tmpStr // degrees
	Variable EqnOfTheCentre = sunCenter(JD) ; sprintf tmpStr,"EqnOfTheCentre:%f;", EqnOfTheCentre ; rtnStr+=tmpStr // dimensionless
	Variable Eccentricity = earthEccentricity(JD) ; sprintf tmpStr,"EccentEarthOrbit:%f;",Eccentricity ;  rtnStr+=tmpStr // dimensionless
	Variable LongOfPerihelion = longitudeOfEarthPerihelion(JD) ; sprintf tmpStr,"LongOfPerihelion:%f;",LongOfPerihelion ; rtnStr+=tmpStr // degrees
	Variable DistanceOfPerihelion = sunDistance(LongOfPerihelion) ; sprintf tmpStr, "DistanceOfPerihelion:%f;",DistanceOfPerihelion ; rtnStr+=tmpStr // meters
	Variable SolarDistance = sunDistance(JD) ; sprintf tmpStr, "SolarDistance:%f;",SolarDistance ; rtnStr+=tmpStr // meters
	Variable ObliquityOfTheEcliptic = trueObliquityOfEcliptic(JD) ; sprintf tmpStr, "ObliquityOfTheEcliptic:%f;",ObliquityOfTheEcliptic ; rtnStr+=tmpStr // degrees
	[Variable SolarRightAscension,Variable SolarDeclination] = sunApparentRADecl(JD) ; sprintf tmpStr, "SolarRightAscension:%f;SolarDeclination:%f;",SolarRightAscension,SolarDeclination ; rtnStr+=tmpStr // radians
	[Variable SolarAzimuth,Variable SolarAltitude] = refractedSunAzimuthAltitude(JD, latVar, lonVar, temp, pressure) ; sprintf tmpStr, "SolarAzimuth:%f;SolarAltitude:%f;",SolarAzimuth,SolarAltitude ; rtnStr+=tmpStr // radia
	Variable SolarRise = sunrise(JD, latVar, lonVar, temp, pressure, altVar) ; sprintf tmpStr, "SolarRise:%f;",SolarRise ; rtnStr+=tmpStr // julian
	Variable SolarSet = sunset(JD, latVar, lonVar, temp, pressure, altVar) ; sprintf tmpStr, "SolarSet:%f;",SolarSet ; rtnStr+=tmpStr // julian
	Variable SolarTransit = sunTransit(JD, latVar, lonVar, temp, pressure) ; sprintf tmpStr, "SolarTransit:%f;",SolarTransit ; rtnStr+=tmpStr // julian
	Variable GoldenHourStart = GoldenHourBegin(JD, latVar, lonVar, temp, pressure, altVar) ; sprintf tmpStr, "GoldenHourStart:%f;",GoldenHourStart ; rtnStr+=tmpStr // julian
	Variable CivilTwilightStart = civilTwilightBegin(JD, latVar, lonVar, temp, pressure, altVar) ; sprintf tmpStr, "CivilTwilightStart:%f;",CivilTwilightStart ; rtnStr+=tmpStr // julian
	Variable CivilTwilightFinish = civilTwilightEnd(JD, latVar, lonVar, temp, pressure, altVar) ; sprintf tmpStr, "CivilTwilightFinish:%f;",CivilTwilightFinish ; rtnStr+=tmpStr // julian
	Variable NauticalTwilightStart = nauticalTwilightBegin(JD, latVar, lonVar, temp, pressure, altVar) ; sprintf tmpStr, "NauticalTwilightStart:%f;",NauticalTwilightStart ; rtnStr+=tmpStr // julian
	Variable NauticalTwilightFinish = nauticalTwilightEnd(JD, latVar, lonVar, temp, pressure, altVar) ; sprintf tmpStr, "NauticalTwilightFinish:%f;",NauticalTwilightFinish ; rtnStr+=tmpStr // julian
	Variable AstronomicalTwilightStart = astronomicalTwilightBegin(JD, latVar, lonVar, temp, pressure, altVar) ; sprintf tmpStr, "AstronomicalTwilightStart:%f;",AstronomicalTwilightStart ; rtnStr+=tmpStr // julian
	Variable AstronomicalTwilightFinish = astronomicalTwilightEnd(JD, latVar, lonVar, temp, pressure, altVar) ; sprintf tmpStr, "AstronomicalTwilightFinish:%f;",AstronomicalTwilightFinish; rtnStr+=tmpStr // julian
	Variable SolarShadow = ShadowLength(SolarAltitude) ; sprintf tmpStr, "SolarShadow:%f;",SolarShadow ; rtnStr+=tmpStr // ratio
	Variable DaylightDuration = (SolarSet - SolarRise)*86400 ; sprintf tmpStr,"DaylightDuration:%f;",DaylightDuration ; rtnStr+=tmpStr // seconds
	Variable Nadir = SolarTransit + 0.5 ; sprintf tmpStr, "Nadir:%f;",Nadir ; rtnStr+=tmpStr // Julian
	[Variable SolarRiseAzimuth,Variable SolarRiseAltitude] = refractedSunAzimuthAltitude(SolarRise, latVar, lonVar, temp, pressure) ; sprintf tmpStr, "SolarRiseAzimuth:%f;",SolarRiseAzimuth ; rtnStr+=tmpStr // radians
	[Variable SolarSetAzimuth, Variable SolarSetAltitude] = refractedSunAzimuthAltitude(SolarSet, latVar, lonVar, temp, pressure) ; sprintf tmpStr, "SolarSetAzimuth:%f;",SolarSetAzimuth ; rtnStr+=tmpStr // radians
	[Variable SolarTransitAzimuth,Variable SolarTransitAltitude] = refractedSunAzimuthAltitude(SolarTransit, latVar, lonVar, temp, pressure) ; sprintf tmpStr, "SolarTransitAltitude:%f;",SolarTransitAltitude; rtnStr+=tmpStr // radians
	Variable SolarZenith = 90 - deg(SolarAltitude) ; sprintf tmpStr, "SolarZenith:%f;",SolarZenith ; rtnStr+=tmpStr // degrees
	Variable SolarRadialSpeed = sunRadialVelocity(JD) ; sprintf tmpStr, "SolarRadialSpeed:%f;",SolarRadialSpeed ; rtnStr+=tmpStr // meters per second
	Variable SolarEquationOfTime = EquationOfTime(JD) ; sprintf tmpStr, "SolarEquationOfTime:%f;",SolarEquationOfTime ; rtnStr+=tmpStr  // radians
	Variable SolarTime = TrueSolarTime(JD, lonVar) ; sprintf tmpStr, "SolarTime:%f;",SolarTime ; rtnStr+=tmpStr  // radians
		
	// sidereal time
	Variable GMST = meanGreenwichSiderealAngle(JD) ; sprintf tmpStr, "GMST:%f;",GMST ; rtnStr+=tmpStr // degrees
	Variable GST = apparentGreenwichSiderealAngle(JD) ; sprintf tmpStr, "GST:%f;",GST ; rtnStr+=tmpStr // degrees
	Variable LMST = meanLocalSiderealAngle(JD,lonVar) ; sprintf tmpStr, "LMST:%f;",LMST ; rtnStr+=tmpStr // degrees
	Variable LST = apparentLocalSiderealAngle(JD,lonVar) ; sprintf tmpStr, "LST:%f;",LST ; rtnStr+=tmpStr // degrees
	
	// equinoxes
	Variable SpringEquin = springEquinox(str2num(JulianToDate(JD,3)[0,3])) ; sprintf tmpStr, "SpringEquin:%f;",SpringEquin ; rtnStr+=tmpStr // julian
	Variable SummerSolst = summerSolstice(str2num(JulianToDate(JD,3)[0,3])) ; sprintf tmpStr, "SummerSolst:%f;",SummerSolst ; rtnStr+=tmpStr // julian
	Variable AutumnEquin = autumnEquinox(str2num(JulianToDate(JD,3)[0,3])) ; sprintf tmpStr, "AutumnEquin:%f;",AutumnEquin ; rtnStr+=tmpStr // julian
	Variable WinterSolst = winterSolstice(str2num(JulianToDate(JD,3)[0,3])) ; sprintf tmpStr, "WinterSolst:%f;",WinterSolst ; rtnStr+=tmpStr // julian
		
	// lunar calculations
	Variable LunarLongitude = moonApparentLongitude(JD) ; sprintf tmpStr, "LunarLongitude:%f;",LunarLongitude ; rtnStr+=tmpStr // degrees
	Variable LunarLatitude = moonLatitude(JD) ; sprintf tmpStr, "LunarLatitude:%f;",LunarLatitude ; rtnStr+=tmpStr // degrees
	Variable LunarDistance = moonDistance(JD) ; sprintf tmpStr, "LunarDistance:%f;", LunarDistance ; rtnStr+=tmpStr // meters
	Variable LunarRadialSpeed = moonRadialVelocity(JD) ; sprintf tmpStr, "LunarRadialSpeed:%f;",LunarRadialSpeed ; rtnStr+=tmpStr // meters per second
	[Variable LunarRightAscension,Variable LunarDeclination] = moonApparentRADecl(JD) ; sprintf tmpStr, "LunarRightAscension:%f;LunarDeclination:%f;",LunarRightAscension,LunarDeclination ; rtnStr+=tmpStr // radians
	[Variable LunarAzimuth,Variable  LunarAltitude] = refractedMoonAzimuthAltitude(JD,latVar,lonVar,temp,pressure) ; sprintf tmpStr, "LunarAzimuth:%f;LunarAltitude:%f;",LunarAzimuth,LunarAltitude ; rtnStr+=tmpStr // radians
	Variable LunarPhaseAngle = moonPhaseAngle(JD) ; sprintf tmpStr, "LunarPhaseAngle:%f;",LunarPhaseAngle ; rtnStr+=tmpStr // radians
	Variable LunarIlluminatedFraction = moonIlluminatedFraction(JD) ; sprintf tmpStr, "LunarIlluminatedFraction:%f;",LunarIlluminatedFraction ; rtnStr+=tmpStr // %
	Variable LunarLimb = moonPositionAngleRelativeToZenith(JD,latVar,lonVar) ; sprintf tmpStr, "LunarLimb:%f;",LunarLimb ; rtnStr+=tmpStr // radians
	Variable LunarSolarAngularSeparation = sunMoonAngularSeparation(JD,latVar,lonVar,temp,pressure) ; sprintf tmpStr, "LunarSolarAngularSeparation:%f;",LunarSolarAngularSeparation ; rtnStr+=tmpStr // radians
	Variable LunarRise = moonrise(JD, latVar, lonVar, temp, pressure, altVar) ; sprintf tmpStr, "LunarRise:%f;",LunarRise ; rtnStr+=tmpStr // julian 
	[Variable LunarRiseAzimuth, Variable LunarRiseAltitude] = refractedMoonAzimuthAltitude(LunarRise,latVar,lonVar,temp,pressure) ; sprintf tmpStr, "LunarRiseAzimuth:%f;",LunarRiseAzimuth ; rtnStr+=tmpStr // radians
	Variable LunarSet = moonset(JD, latVar, lonVar, temp, pressure, altVar) ; sprintf tmpStr, "LunarSet:%f;",LunarSet ; rtnStr+=tmpStr // julian
	[Variable LunarSetAzimuth,Variable LunarSetAltitude] = refractedMoonAzimuthAltitude(LunarSet,latVar,lonVar,temp,pressure) ; sprintf tmpStr, "LunarSetAzimuth:%f;",LunarSetAzimuth ; rtnStr+=tmpStr // radians
	Variable LunarTransit = moonTransit(JD, latVar, lonVar, temp, pressure) ; sprintf tmpStr, "LunarTransit:%f;",LunarTransit ; rtnStr+=tmpStr // julian
	[Variable LunarTransitAzimuth,Variable LunarTransitAltitude] = refractedMoonAzimuthAltitude(LunarTransit,latVar,lonVar,temp,pressure) ; sprintf tmpStr, "LunarTransitAltitude:%f;",LunarTransitAltitude ; rtnStr+=tmpStr // radians
	Variable LunarPerigee = moonPerigee(JD) ; sprintf tmpStr, "LunarPerigee:%f;",LunarPerigee ; rtnStr+=tmpStr // julian
	Variable LunarPerigeeDistance = moonDistance(LunarPerigee) ; sprintf tmpStr, "LunarPerigeeDistance:%f;",LunarPerigeeDistance ; rtnStr+=tmpStr // meters 
	Variable LunarApogee = moonApogee(JD) ; sprintf tmpStr, "LunarApogee:%f;",LunarApogee ; rtnStr+=tmpStr // julian
	Variable LunarApogeeDistance = moonDistance(LunarApogee) ; sprintf tmpStr, "LunarApogeeDistance:%f;",LunarApogeeDistance ; rtnStr+=tmpStr // meters
	Variable LunarAscendingNode = moonMeanLongitudeOfAscendingNode(JD) ; sprintf tmpStr, "LunarAscendingNode:%f;",LunarAscendingNode ; rtnStr+=tmpStr // radians
	Variable NewLune = newMoon(JD) ; sprintf tmpStr, "NewLune:%f;",NewLune ; rtnStr+=tmpStr // julian
	Variable FirstQuarterLune = firstQuarterMoon(JD) ; sprintf tmpStr, "FirstQuarterLune:%f;",FirstQuarterLune ; rtnStr+=tmpStr // julian
	Variable FullLune = fullMoon(JD) ; sprintf tmpStr, "FullLune:%f;",FullLune ; rtnStr+=tmpStr // julian
	Variable LastQuarterLune = lastQuarterMoon(JD) ; sprintf tmpStr, "LastQuarterLune:%f;",LastQuarterLune ; rtnStr+=tmpStr // julian
	Variable LunarShadow = ShadowLength(LunarAltitude) ; sprintf tmpStr, "LunarShadow:%f;",LunarShadow ; rtnStr+=tmpStr // ratio
	Variable LunarAge = getMoonAge(datStr,timeStr,TimeZone) ; sprintf tmpStr,"LunarAge:%f;",LunarAge ; rtnStr+=tmpStr // days
	String LunarPhase = getMoonPhase(LunarAge) ; sprintf tmpStr,"LunarPhase:%s;",LunarPhase ; rtnStr+=tmpStr // string
	Variable NightDuration = 86400 - DaylightDuration ; sprintf tmpStr,"NightDuration:%f;",NightDuration ; rtnStr+=tmpStr // seconds
	Variable LunarZenith = 90 - deg(LunarAltitude) ; sprintf tmpStr,"LunarZenith:%f;",LunarZenith ; rtnStr+=tmpStr // seconds

	#ifdef webApps
		// location of the poles
		[Variable NPlatVar, Variable NPlonVar] = getNDipPole(str2num(JulianToDate(JD,3)[0,3]))  ; sprintf tmpStr,"NPlatitude:%f;NPlongitude:%f;",NPlatVar,NPlonVar ; rtnStr+=tmpStr // seconds
		[Variable SPlatVar, Variable SPlonVar] = getSDipPole(str2num(JulianToDate(JD,3)[0,3]))  ; sprintf tmpStr,"SPlatitude:%f;SPlongitude:%f;",SPlatVar,SPlonVar ; rtnStr+=tmpStr // seconds
	#endif

	return rtnStr

end
//___________________________________________________________________________________________________	
// function to return a list of keys from a Key pair list (keyLst), given the key-pair separator (keysepStr) and list separator (lstSepStr)
Static Function/S getKeysfromKeyPairs(keyLst,keysepStr,lstSepStr)
	String keyLst,keysepStr,lstSepStr

	String rtnStr="",keyStr
	Variable i
	
	do
		SplitString/E=".*"+keysepstr StringFromList(i,keyLst,lstsepStr)
		rtnStr+=ReplaceString(keysepStr,S_value,lstsepStr)
		i+=1
	while (i<ItemsInList(keyLst))
	
	return rtnStr	
end
//___________________________________________________________________________________________________	
// funtion to return the the key in a keypair given the number variable 
Static Function/S KeyByNumber(numVar,kwListStr [, keySepStr, listSepStr])
	Variable numVar
	String kwListStr,keySepStr,listSepStr
	
	// Default key separator 
	if (ParamIsDefault(keySepStr))
		keySepStr=":"
	endif
	
	// default list separator
	if (ParamIsDefault(listSepStr))
		listSepStr=";"
	endif
		
	Variable i
	do 
		String keyPairStr = StringFromList(i, kwListStr , listSepStr) 
		
		String keyStr,matchVarStr
		Splitstring/E="(.*)"+keySepStr+"(.*)" keyPairStr,keyStr,matchVarStr
		
		if (numVar==str2num(matchVarStr))
			return keyStr
		else
			i+=1
		endif
	while (i<ItemsInList(kwListStr))
end
//___________________________________________________________________________________________________	
// List of timezones and military letter
Static Function/S TimeZoneKeyPairs()
	
	String rtnStr=""
	rtnStr+="UTC −12:00 (X),-12;"
	rtnStr+="UTC −11:00 (X),-11;"
	rtnStr+="UTC −10:00 (W),-10;"
	rtnStr+="UTC −09:30 (V),-9.5;"
	rtnStr+="UTC −09:00 (V),-9;"
	rtnStr+="UTC −08:00 (U),-8;"
	rtnStr+="UTC −07:00 (T),-7;"
	rtnStr+="UTC −06:00 (S),-6;"
	rtnStr+="UTC −05:00 (R),-5;"
	rtnStr+="UTC −04:00 (Q),-4;"
	rtnStr+="UTC −03:30 (P),-3.5;"
	rtnStr+="UTC −03:00 (P),-3;"
	rtnStr+="UTC −02:00 (O),-2;"
	rtnStr+="UTC −01:00 (N),-1;"
	rtnStr+="UTC ±00:00 (Z),0;"
	rtnStr+="UTC +01:00 (A),1;"
	rtnStr+="UTC +02:00 (B),2;"
	rtnStr+="UTC +03:00 (C),3;"
	rtnStr+="UTC +03:30 (C),3.5;"
	rtnStr+="UTC +04:00 (D),4;"
	rtnStr+="UTC +04:30 (D),4.5;"
	rtnStr+="UTC +05:00 (E),5;"
	rtnStr+="UTC +05:30 (E),5.5;"
	rtnStr+="UTC +05:45 (E),5.75;"
	rtnStr+="UTC +06:00 (F),6;"
	rtnStr+="UTC +06:30 (F),6.5;"
	rtnStr+="UTC +07:00 (G),7;"
	rtnStr+="UTC +08:00 (H),8;"
	rtnStr+="UTC +08:45 (H),8.75;"
	rtnStr+="UTC +09:00 (I),9;"
	rtnStr+="UTC +09:30 (I),9.5;"
	rtnStr+="UTC +10:00 (K),10;"
	rtnStr+="UTC +10:30 (K),10.5;"
	rtnStr+="UTC +11:00 (L),11;"
	rtnStr+="UTC +12:00 (M),12;"
	rtnStr+="UTC +12:45 (M),12.75;"
	rtnStr+="UTC +13:00 (M),13;"
	rtnStr+="UTC +14:00 (M),14;"
	
	return rtnStr
end
//___________________________________________________________________________________________________
// function to parse a date string
Static Function/S ParseDateStringNum(DateStr)
    String datestr          // Format: yyymmdd

    String year, monthStr, day
      
    sscanf dateStr, "%4s%2s%2s", year, monthStr, day
   
    // switch on the month numeric string
    Strswitch(monthStr)
        case "01":
            monthStr="jan"
            break
        case "02":
            monthStr="feb"
            break
        case "03":
            monthStr="mar"
            break
        case "04":
            monthStr="apr"
            break
        case "05":
            monthStr="may"
            break
        case "06":
            monthStr="jun"
            break
        case "07":
            monthStr="jul"
            break
        case "08":
            monthStr="aug"
            break
        case "09":
            monthStr="sep"
            break
        case "10":
            monthStr="oct"
            break
        case "11":
            monthStr="nov"
            break
        case "12":
            monthStr="dec"
            break
    endswitch
	
	DateStr = year+"-"+monthStr+"-"+day // yyyy-mmm-dd
	Return dateStr  
  
End
//___________________________________________________________________________________________________	
// function to parse date string 
Static Function/S ParseDateStringAlpha(datestr)
    String datestr          // Format: yyyy-mmm-dd

    String yearStr, monthStr, dayStr
   
    sscanf datestr, "%4s-%3s-%2s", yearStr, monthStr, dayStr
   
   // switch on the month string
    strswitch(monthStr)
        case "jan":
            monthStr="01"
            break
        case "feb":
            monthStr="02"
            break
        case "mar":
            monthStr="03"
            break
        case "apr":
            monthStr="04"
            break
        case "may":
            monthStr="05"
            break
        case "jun":
            monthStr="06"
            break
        case "jul":
            monthStr="07"
            break
        case "aug":
            monthStr="08"
            break
        case "sep":
            monthStr="09"
            break
        case "oct":
            monthStr="10"
            break
        case "nov":
            monthStr="11"
            break
        case "dec":
            monthStr="12"
            break
    endswitch
	
	DateStr = yearStr+monthStr+dayStr // yyyymmdd
	Return dateStr  
  
End
//___________________________________________________________________________________________________
