﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.
#include "AstronomicalAlgorithms" // requires MJD

//     1) Moyer, T.D., Transformation from Proper Time on Earth to 
//        Coordinate Time in Solar System Barycentric Space-Time Frame
//        of Reference, Parts 1 and 2, Celestial Mechanics 23 (1981),
//        33-56 and 57-68.
//
//     2) Moyer, T.D., Effects of Conversion to the J2000 Astronomical
//        Reference System on Algorithms for Computing Time Differences
//        and Clock Rates, JPL IOM 314.5--942, 1 October 1985.

Static Constant DELTA_T_A = 32.184
Static constant K = 1.657E-3
Static Constant EB = 1.671E-2
Static Constant M0  = 6.239996E0
Static Constant M1 = 1.99096871E-7

//___________________________________________________________________________________________________
// The difference between ET ephemeris time and UTC is given by:
Function DELTA_ET(Variable JD)
	Variable E = EccentricAnomaly(JD)

	return DELTA_T_A + K*sin(E) + DELTA_AT(JD)
end
//___________________________________________________________________________________________________
Function EccentricAnomaly(Variable JD)

	Variable M = MeanAnomaly(JD)
	
	Return M + EB*sin(M)
end
//___________________________________________________________________________________________________
Function MeanAnomaly(Variable JD)	
	Variable t = (JD - 2451545.0) * 86400
	
	return M0 + (M1 * t)
end
//___________________________________________________________________________________________________
// Difference between TAI atomic time and UTC
// Number of UTC leap seconds from julian day - this must be updated manually
Function DELTA_AT(variable JD)

	if ((JD >= DateToJulian(1961,01,01))&&(JD < DateToJulian(1961,08,01))) //
		Return 1.4228180 + (MJD(JD) - 37300.) * 0.001296
	elseif ((JD >= DateToJulian(1961,08,01))&&(JD < DateToJulian(1962,01,01))) //
		Return 1.3728180 + (MJD(JD) - 37300.) * 0.001296
	elseif ((JD <= DateToJulian(1962,01,01))&&(JD < DateToJulian(1963,11,01))) //
		Return 1.8458580 + (MJD(JD) - 37665.) * 0.0011232
	elseif ((JD >= DateToJulian(1963,11,01))&&(JD < DateToJulian(1964,01,01))) //
		Return 1.9458580 + (MJD(JD) - 37665.) * 0.0011232
	elseif ((JD >= DateToJulian(1964,01,01))&&(JD < DateToJulian(1964,04,01))) //
		Return 3.2401300 + (MJD(JD) - 38761.) * 0.001296
	elseif ((JD >= DateToJulian(1964,04,01))&&(JD < DateToJulian(1964,09,01))) //
		Return 3.3401300 + (MJD(JD) - 38761.) * 0.001296
	elseif ((JD >= DateToJulian(1964,09,01))&&(JD < DateToJulian(1965,01,01))) //
		Return 3.4401300 + (MJD(JD) - 38761.) * 0.001296
	elseif ((JD >= DateToJulian(1965,01,01))&&(JD < DateToJulian(1965,03,01))) //
		Return 3.5401300 + (MJD(JD) - 38761.) * 0.001296
	elseif ((JD >= DateToJulian(1965,03,01))&&(JD < DateToJulian(1965,07,01))) //
		Return 3.6401300 + (MJD(JD) - 38761.) * 0.001296
	elseif ((JD >= DateToJulian(1965,07,01))&&(JD < DateToJulian(1965,09,01))) //
		Return 3.7401300 + (MJD(JD) - 38761.) * 0.001296
	elseif ((JD >= DateToJulian(1965,09,01))&&(JD < DateToJulian(1966,01,1))) //
		Return 3.8401300 + (MJD(JD) - 38761.) * 0.001296
	elseif ((JD >= DateToJulian(1966,01,1))&&(JD < DateToJulian(1968,02,01))) //
		Return 4.3131700 + (MJD(JD) - 39126.) * 0.002592
	elseif ((JD >= DateToJulian(1968,02,01))&&(JD < DateToJulian(1972,01,01))) //
		Return 4.2131700 + (MJD(JD) - 39126.) * 0.002592
	elseif ((JD <= DateToJulian(1972,01,01))&&(JD < DateToJulian(1972,07,01))) //
		Return 10
	elseif ((JD >= DateToJulian(1972,07,01))&&(JD < DateToJulian(1973,01,01)))	//
		Return 11
	elseif ((JD >= DateToJulian(1973,01,01))&&(JD < DateToJulian(1974,01,01))) //
		Return 12
	elseif ((JD >= DateToJulian(1974,01,01))&&(JD < DateToJulian(1975,01,01))) //
		Return 13
	elseif ((JD >= DateToJulian(1975,01,01))&&(JD < DateToJulian(1976,01,01))) //
		Return 14
	elseif ((JD >= DateToJulian(1976,01,01))&&(JD < DateToJulian(1977,01,01))) //
		Return 15
	elseif ((JD >= DateToJulian(1977,01,01))&&(JD < DateToJulian(1978,01,01))) //
		Return 16
	elseif ((JD >= DateToJulian(1978,01,01))&&(JD < DateToJulian(1979,01,01))) //
		Return 17
	elseif ((JD >= DateToJulian(1978,01,01))&&(JD < DateToJulian(1980,01,01))) //
		Return 18
	elseif ((JD >= DateToJulian(1980,01,01))&&(JD < DateToJulian(1981,07,01))) //
		Return 19
	elseif((JD >= DateToJulian(1981,07,01))&&(JD < DateToJulian(1982,07,01))) //
		Return 20
	elseif ((JD >= DateToJulian(1982,07,01))&&(JD < DateToJulian(1983,07,01))) //
		Return 21
	elseif ((JD >= DateToJulian(1983,01,01))&&(JD < DateToJulian(1985,07,01))) //
		Return 22
	elseif ((JD >= DateToJulian(1985,07,01))&&(JD < DateToJulian(1988,01,01))) //
		Return 23
	elseif ((JD >= DateToJulian(1988,01,01))&&(JD < DateToJulian(1990,01,01))) //
		Return 24
	elseif ((JD >= DateToJulian(1990,01,01))&&(JD < DateToJulian(1991,01,01))) //
		Return 25
	elseif ((JD >= DateToJulian(1991,01,01))&&(JD < DateToJulian(1992,07,01))) //
		Return 26
	elseif ((JD >= DateToJulian(1992,07,01))&&(JD < DateToJulian(1993,07,01))) //
		Return 27
	elseif ((JD >= DateToJulian(1993,07,01))&&(JD < DateToJulian(1994,07,01))) //
		Return 28
	elseif ((JD >= DateToJulian(1994,07,01))&&(JD < DateToJulian(1996,01,01))) //
		Return 29
	elseif ((JD >= DateToJulian(1996,01,01))&&(JD < DateToJulian(1997,07,01))) //
		Return 30
	elseif ((JD >= DateToJulian(1997,07,01))&&(JD < DateToJulian(1999,01,01))) //
		Return 31
	elseif ((JD >= DateToJulian(1999,01,01))&&(JD < DateToJulian(2006,01,01))) //
		Return 32
	elseif ((JD >= DateToJulian(2006,01,01))&&(JD < DateToJulian(2009,01,01))) //
		Return 33
	elseif ((JD >= DateToJulian(2009,01,01))&&(JD < DateToJulian(2012,07,01))) //
		Return 34
	elseif ((JD >= DateToJulian(2012,07,01))&&(JD < DateToJulian(2015,07,01))) //
		Return 35
	elseif ((JD >= DateToJulian(2015,07,01))&&(JD < DateToJulian(2017,01,01))) //
		Return 36
	elseif (JD >= DateToJulian(2017,01,01)) //
		Return 37
	endif
	
	Return 0
end
//___________________________________________________________________________________________________
// Get delta Terestrial Dynamical Time from Universal time, dT = TDT - UT
// https://eclipse.gsfc.nasa.gov/SEhelp/deltatpoly2004.html
 Function deltaT(Variable JD)

	Variable year = str2num(JulianToDate(JD,3)[0,3])
	Variable y = year + ((JD - DateToJulian(year,01,01) - 0.5)/ 365.25)
	Variable c = -0.000012932 * (y - 1955)^2
	Variable u, t, dT
	
	if (year <=  -500 )
		u = (year - 1820)/100
		dT = -20 + 32*u&2
	elseif ((year >= -500 )&&(year < 500 ))
		u = y/100
		dT = 10583.6 - 1014.41 * u + 33.78311 * u^2 - 5.952053 * u^3 - 0.1798452 * u^4 + 0.022174192 * u^5 + 0.0090316521 * u^6 
	elseif ((year >= 500 )&&(year < 1600 ))
		u = (y - 1000)/100
		dT = 1574.2 - 556.01 * u + 71.23472 * u^2 + 0.319781 * u^3 - 0.8503463 * u^4 - 0.005050998 * u^5 + 0.0083572073 * u^6
	elseif ((year >= 1600 )&&(year < 1700 ))
		t = y - 1600
		dT = 120 - 0.9808 * t - 0.01532 * t^2 + t^3 / 7129
	elseif ((year >= 1700 )&&(year < 1800 ))
		t = y - 1700
		dT = 8.83 + 0.1603 * t - 0.0059285 * t^2 + 0.00013336 * t^3 - t^4 / 1174000
	elseif ((year >= 1800 )&&(year < 1860 ))
		t = y - 1800
		dT = 13.72 - 0.332447 * t + 0.0068612 * t^2 + 0.0041116 * t^3 - 0.00037436 * t^4 	+ 0.0000121272 * t^5 - 0.0000001699 * t^6 + 0.000000000875 * t^7
	elseif ((year >=  1860 )&&(year <  1900 ))
		t = y - 1860
		dT = 7.62 + 0.5737 * t - 0.251754 * t^2 + 0.01680668 * t^3 -0.0004473624 * t^4 + t^5 / 233174
	elseif ((year >= 1900 )&&(year < 1920 ))
		t = y - 1900
		dT = -2.79 + 1.494119 * t - 0.0598939 * t^2 + 0.0061966 * t^3 - 0.000197 * t^4
	elseif ((year >= 1920 )&&(year < 1941 ))
		t = y - 1920
		dT = 21.20 + 0.84493*t - 0.076100 * t^2 + 0.0020936 * t^3
	elseif ((year >= 1941 )&&(year < 1961 ))
		t = y - 1950
		dT = 29.07 + 0.407*t - t^2/233 + t^3 / 2547
	elseif ((year >= 1961 )&&(year < 1986 ))
		t = y - 1975
		dT = 45.45 + 1.067*t - t^2/260 - t^3 / 718
	elseif ((year >= 1986 )&&(year < 2005 ))
		t = y - 2000
		dt = 63.86 + 0.3345 * t - 0.060374 * t^2 + 0.0017275 * t^3 + 0.000651814 * t^4 + 0.00002373599 * t^5
	elseif ((year >= 2005 )&&(year < 2050 ))
		 t = y - 2000
		 dT = 62.92 + 0.32217 * t + 0.005589 * t^2
	elseif ((year >= 2050 )&&(year < 2150 ))
		dt = -20 + 32 * ((y-1820)/100)^2 - 0.5628 * (2150 - y)
	elseif (year >= 2150 )
		u = (year - 1820) / 100
		dT = -20 + 32 * u^2
	endif
	
	return dT + c
end
//___________________________________________________________________________________________________
// Get delta Terestrial Dynamical Time from Universal time, dT = TDT - UT	
// Jean Meeus, Astronomical Algorithms (second edition), chapter 10, p77
//Function deltaT(Variable JD) // Imprecise
	
	Variable year = str2num(JulianToDate(JD,3)[0,3]) 
	year+= ((JD - DateToJulian(year,01,01) -0.5)/ 365.25)
	
	Variable dT
	variable t = (year - 2000)/100
	Variable phi = (year - 1900)/100
	
	if (year < 947) // Meeus 10.1
		dT = 2177 + 497*t + 44.1 *t^2
	elseif ((year >= 948) && (year  < 1599))
		dT = 102 + 102*t + 25.3*t^2 // meeus 10.2
	elseif ((year >= 1800) && (year  < 1899)) // Meeus ch10
		dt = -2.5 + 288.95*phi + 5218.61*phi^2 + 56282.84*phi^3 + \
			342011.78*phi^4 + 1061660.75*phi^5 + 2087298.89*phi^6 + \
			2513807.78*phi^7 + 1818961.41*phi^8 + 727058.63*phi^9 + \
			123563.93*phi^10
	elseif ((year >= 1900) && (year  < 1997)) // Meeus ch10
		dT = -2.44 + 87.24*phi + 815.2*phi^2 - 2637.8*phi^3 - \
			18756.33*phi^4 + 124906*phi^5 - 303191.19*phi^6 + \
			372919.88*phi^7 - 232424.66*phi^8 + 58353.42*phi^9
	elseif ((year >= 2000) && (year  < 2100)) // Meeus ch10
		dT = (102 + 102*t + 25.3*t^2)  + (0.37 * (year - 2100))
	else
		dT = 0
	endif

	// in seconds 
	return dT
end